-- ucFirst :: string -> string
function ucFirst(s)
  return s:gsub("^%l", string.upper)
end

-- upperCaseWordsHelper :: string -> string -> string
local function upperCaseWordsHelper(x, xs)
  return x:upper() .. xs:lower()
end

-- ucWords :: string -> string
function ucWords(s)
  return s:gsub("(%a)([%w_']*)", upperCaseWordsHelper)
end

-- underscoreToSpace :: string -> string
function underscoreToSpace(s)
  return s:gsub("_", " ")
end

-- doubleUnderscoreToApostrophe :: string -> string
function doubleUnderscoreToApostrophe(s)
	return s:gsub("__", "'")
end

-- fixGender :: Gender -> string -> string
function fixGender(gender, s)
  return xir(gender, xim(gender, (xhe(gender, s))))
end

-- xir :: Gender -> string -> string
function xir(gender, s)
  if     gender ==        "male" then return s:gsub("xir",        "his") 
  elseif gender ==      "female" then return s:gsub("xir",        "her")
  elseif gender == "androgynous" then return s:gsub("xir", "his or her")
  else                                return s
	end
end

-- xim :: Gender -> string -> string
function xim(gender, s)
  if     gender ==        "male" then return s:gsub("xim",        "him") 
  elseif gender ==      "female" then return s:gsub("xim",        "her")
  elseif gender == "androgynous" then return s:gsub("xim", "him or her")
  else                                return s
	end
end

-- xhe :: Gender -> string -> string
function xhe(gender, s)
  if     gender ==        "male" then return s:gsub("xhe",        "he") 
  elseif gender ==      "female" then return s:gsub("xhe",       "she")
  elseif gender == "androgynous" then return s:gsub("xhe", "he or she")
  else                                return s
	end
end

-- aOrAn :: string -> string
function aOrAn(word)
  if string.find(string.sub(word, 1,1), "[AEIOUaeiou]") then return "an"
  else return "a" end
end

-- combine :: string -> string
function combine(x,y)
  return (x or "") .. (y or "")
end

-- combineWithSpace :: string -> string
function combineWithSpace(x,y)
  if      x == nil or x == "" then  return y
  elseif  y == nil or y == "" then  return x
  else                              return x .. " " .. y end
end

--[[ KeyValuePair
function KeyValuePair(k, v)
	return { key = k, value = v }
end
]]--

-- log :: a -> a
function log(x)
  print(x)
  return x
end

-- logTable :: { k -> a } -> { k -> a }
function logTable(t)
  for k,v in pairs(t) do
    print("(" .. k .. ", " .. v .. ")")
  end
  return t
end

--[[ liftToNilMonad
function liftToNilMonad(f)
  return 
    function(x)
      if x == nil then
        return nil
      else
        return f(x)
      end
    end  
end
]]--

--[[ errorMonadic
function errorMonadic(x, funcs)
  local y = x
  return foldr(
    function(k, f)
      if k == nil then
        return nil
      else
       return f(k) -- <-- function on k, k should be a table.
      end
    end,
    y,
    funcs
  )
end
]]--

-- eitherOr :: a -> a -> a
function eitherOrRandom(x, y)
  return eitherOr(random(1,2) == 1, x, y)
end

-- eitherOrP :: bool -> a -> a -> a
function eitherOr(p, x, y)
  if p then
    return x
  else
    return y
  end
end

-- randomFrom :: { a } -> a
-- Implicitly, if list is empty, nil will be returned.
function randomFrom(list)
	local c = count(list)
	for _,v in pairs(list) do
		rand = random(1, c)
		if rand == 1 then
			return v 
		else
			c = c - 1
		end
	end
end

-- uniqueRandomsFrom :: int -> { a } -> { a }
-- Returns between 1 and maxReturns!
function uniqueRandomsFrom(maxReturns, list)
  if maxReturns <= 0 then return { } end
  
  local output = { }
	local c = count(list)
	for _,v in pairs(list) do
		rand = random(1, c)
		if rand == 1 then
			table.insert(output, v) -- This line is changed to hopefully fix the bug >.>
      maxReturns = maxReturns - 1
      if maxReturns <= 0 then
        return output
      end
		else  -- Else clause jumbles up the c variable, this guarantees that there could be less returns than maxReturns.
			c = c - 1
		end
	end
  return output
end

-- uniqueRandomsFromCartesian :: { a } -> { b } -> a,b
-- (x, y) | x <- xs and y <- ys and x ~= y 
function uniqueRandomsFromCartesian(xs, ys)
  local x = randomFrom(xs)
  local y = randomFrom(ys)
  while x == y and y ~= nil do -- This can loop infinitely.
    y = randomFrom(ys)
  end
  return x, y
end

-- End of File.