-- Checking traits to be valid will be postponed until a later version.

function rollName()
  characterTemplate.setName(
    underscoreToSpace(
			doubleUnderscoreToApostrophe(
        generateRandomName(
          characterTemplate
				)
      )
    )
  )
end

function rollRace()
  characterTemplate.setRace(
		generateRandomRace("any")
	)
	-- Also roll the Tribe to make sure it doesn't get invalid.
	rollTribe()
end

function rollTribe()
	characterTemplate.setTribe(
		generateRandomTribe(
			characterTemplate
		)
	)
end

function rollGender()
  characterTemplate.setGender(
		generateRandomGender()
	)
end

function rollClass()
  characterTemplate.setClass(
    generateRandomClass(
      characterTemplate,
      nil
    )
  )
end

function rollAge()
  characterTemplate.setAge(
    generateRandomAge("any")
  )
end

function rollAlignment()
  characterTemplate.setAlignment(
    generateRandomAlignment(
			characterTemplate
    )
  )
end

function rollFeature()
  characterTemplate.setFeature(
    generateRandomFeature(
      characterTemplate
    )
  )
end

function rollFaith()
  characterTemplate.setFaith(
    generateRandomFaith(
      characterTemplate
    )
  )
end

-- Function too complicated. Consider simplifying.
function rollMotivation(num)
  local motivation = generateRandomMotivation(
    characterTemplate)
  
  -- If no options can be found, it might be impossible to find unique traits.
  -- Is this even still relevant?
	-- We can use nil-monadic operations to do this?
  if motivation == "Error: No options found." then
    characterTemplate.setMotivation(num, motivation)
  else
  
  local a, b = unpack(filter(function(x) return x ~= num end, { 1,2,3 }))
  
    -- Function 2
    while 
      motivation == characterTemplate.getMotivation(a) or 
      motivation == characterTemplate.getMotivation(b)
		do
			motivation = generateRandomMotivation(
				characterTemplate)
    end
    -- End of Function 2
    
    characterTemplate.setMotivation(num, motivation)
  end
end

function rollAllMotivations()
  local motivations = generateRandomMotivations(
    characterTemplate
	)
  characterTemplate.setMotivation(1, motivations[1] or "Error: No options found.")
  characterTemplate.setMotivation(2, motivations[2] or "")  -- Finding less than three motivations is fine, as long as at least one motivation is found.
  characterTemplate.setMotivation(3, motivations[3] or "")
end

local function generateCharacterString(template)
	return
		foldr(
			combineWithSpace,
			"",
			{
				"||",
				ucWords(template.getName()),
				"is",
				aOrAn(template.getAge()),
				template.getAge(),
				ucWords(template.getTribe()),
				ucWords(template.getRace()),
				template.getGender(),
				"that appears to be",
				aOrAn(template.getClass()),
				template.getClass(),
				"and",
				template.getFeature()
			}
		)
	.. "."
end

function printToChat(self)
	local charString = generateCharacterString(characterTemplate)
	--print(strlen(charString))
  SendChatMessage(charString, "EMOTE")
end

function rollAll(self)
  local activeModules = { rollRace, rollTribe, rollAge, rollClass, rollGender, rollName, rollAlignment, rollFaith, rollFeature, rollAllMotivations }
  invokeAll(activeModules)
end

-- End of File